<?php
namespace Ak\Test\Constraint\Container;

use PHPUnit\Framework\Constraint\Constraint;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;

/**
 * Utvrđuje da container sadrži servise koji su isntance date klase.
 *
 * @package Ak\Test\Constraint\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/test
 */
class ServiceIsInstanceOf extends Constraint
{
    private array $names;
    private string $lastName;
    private string $instanceOf;

    /**
     * Konstruktor.
     *
     * @param array|string $names
     * @param string $instanceOf
     */
    public function __construct($names, string $instanceOf)
    {
        $this->names = !is_array($names) ? [$names] : $names;
        $this->instanceOf = $instanceOf;
    }

    /**
     * @param ContainerInterface $other
     *
     * @return bool
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function matches($other): bool
    {
        foreach ($this->names as $name) {
            $this->lastName = $name;
            if (!$other->get($name) instanceof $this->instanceOf) {
                return false;
            }
        }

        return true;
    }

    /**
     * @param mixed $other
     * @return string
     */
    protected function failureDescription($other): string
    {
        return 'the service with the name "' . $this->lastName . '" ' . $this->toString();
    }

    /**
     * Returns a string representation of the object.
     */
    public function toString(): string
    {
        return 'is an instance of "' . $this->instanceOf . '"';
    }
}
