<?php
namespace Ak\Test\Constraint\Container;

use PHPUnit\Framework\Constraint\Constraint;
use Psr\Container\ContainerExceptionInterface;
use Psr\Container\ContainerInterface;
use Psr\Container\NotFoundExceptionInterface;

/**
 * Utvrđuje da li sva data imena pokazuju na istu instancu servisa.
 *
 * @package Ak\Test\Constraint\Container
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/test
 */
class ServiceIsShared extends Constraint
{
    private array $names;
    private string $lastName;

    /**
     * Konstruktor.
     *
     * @param array|string $names
     */
    public function __construct($names)
    {
        $this->names = !is_array($names) ? [$names] : $names;
    }

    /**
     * @param ContainerInterface $other
     *
     * @return bool
     * @throws ContainerExceptionInterface
     * @throws NotFoundExceptionInterface
     */
    public function matches($other): bool
    {
        $service = $other->get(array_shift($this->names));

        foreach ($this->names as $name) {
            $this->lastName = $name;
            if ($service !== $other->get($name)) {
                return false;
            }
        }

        return true;
    }

    /**
     * @param mixed $other
     * @return string
     */
    protected function failureDescription($other): string
    {
        return 'the given container ' . $this->toString();
    }

    /**
     * Returns a string representation of the object.
     */
    public function toString(): string
    {
        return 'has a shared service named "' . $this->lastName . '"';
    }
}
